"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hook = exports.HookEvent = void 0;
const tslib_1 = require("tslib");
const crypto_1 = require("crypto");
const sequelize_1 = require("sequelize");
const axios_1 = (0, tslib_1.__importDefault)(require("axios"));
const pino_1 = (0, tslib_1.__importDefault)(require("pino"));
const db_1 = require("./db");
const user_1 = require("./user");
const logger = (0, pino_1.default)();
var HookEvent;
(function (HookEvent) {
    HookEvent["PackageStar"] = "package:star";
    HookEvent["PackageUnstar"] = "package:unstar";
    HookEvent["PackagePublish"] = "package:publish";
    HookEvent["PackageUnpublish"] = "package:unpublish";
    HookEvent["PackageOwner"] = "package:owner";
    HookEvent["PackageOwnerRm"] = "package:owner-rm";
    HookEvent["PackageDistTag"] = "package:dist-tag";
    HookEvent["PackageDistTagRm"] = "package:dist-tag-rm";
    HookEvent["PackageDeprecated"] = "package:deprecated";
    HookEvent["PackageUndeprecated"] = "package:undeprecated";
    HookEvent["PackageChange"] = "package:change";
})(HookEvent = exports.HookEvent || (exports.HookEvent = {}));
async function postHook(hook, payload) {
    const secret = hook.getDataValue("secret");
    const stringPayload = JSON.stringify(payload);
    const signature = (0, crypto_1.createHmac)("sha256", secret)
        .update(stringPayload)
        .digest("hex");
    try {
        const response = await axios_1.default.post(hook.getDataValue("endpoint"), {
            json: {
                payload,
            },
            responseType: "json",
        }, {
            headers: {
                "x-npm-signature": `sha256=${signature}`,
            },
        });
    }
    catch (err) {
        logger.error({ err, hook }, "Error calling webhook");
    }
}
class Hook extends sequelize_1.Model {
    static getUserHooks(ownerId) {
        return Hook.findAll({ where: { ownerId } });
    }
    // Very simple and slow notification system
    // TODO: Refactor this call so that it can be re-implemented
    // more robostly using a queue system or similar.
    // TODO: A user may have lost rights while the hook was already
    // created, so we could lazily check the rights here and remove
    // thoose hooks...
    static async notifyHooks(type, name, payload) {
        const query = { where: { type, name } };
        // 1) Find all hooks
        const hooks = await Hook.findAll(query);
        // 2) Call the endpoints
        await Promise.all(hooks.map((hook) => postHook(hook, payload)));
        // 3) Update hooks stats
        await Hook.update({ triggerCount: (0, sequelize_1.literal)("triggerCount + 1"), lastTriggerAt: new Date() }, query);
    }
}
exports.Hook = Hook;
Hook.init({
    id: {
        type: sequelize_1.DataTypes.UUID,
        defaultValue: sequelize_1.DataTypes.UUIDV4,
        primaryKey: true,
    },
    type: sequelize_1.DataTypes.ENUM("package", "owner", "scope"),
    name: sequelize_1.DataTypes.STRING,
    endpoint: sequelize_1.DataTypes.STRING,
    secret: {
        type: sequelize_1.DataTypes.STRING,
    },
    lastTriggeredAt: sequelize_1.DataTypes.DATE,
    triggerCount: sequelize_1.DataTypes.NUMBER,
}, {
    sequelize: db_1.db,
    modelName: "Hook",
});
user_1.User.hasMany(Hook, { foreignKey: "ownerId" });
//# sourceMappingURL=hook.js.map